@@ -112,7 +112,7 @@ setMethod("show", "Cov", function(object){
     print(object@call)
     cat("-> Method: ", object@method, "\n")
     if(is.list(object@singularity))
-        cat(strwrap(robustbase:::singularityMsg(object@singularity, object@n.obs)), sep ="\n")
+        cat(strwrap(.MCDsingularityMsg(object@singularity, object@n.obs)), sep ="\n")
 
     digits = max(3, getOption("digits") - 3)
     cat("\nEstimate of Location: \n")
