@@ -126,6 +126,6 @@ PcaCov.default <- function(x, k=0, kmax=ncol(x), cov.control = CovControlMcd(),
                             n.obs=n)
 
     ## Compute distances and flags
-    res <- rrcov:::.distances(x, p, res)
+    res <- pca.distances(res, x, p)
     return(res)
 }
